<?php

namespace AltSolution\Admin\Form;


abstract class AbstractField extends AbstractComponent implements FieldInterface
{
    protected $viewNs = 'admin::form.field';

    /**
     * List of predefined validation rules
     * @var array
     */
    protected $rules = [];

    /**
     * @return array
     */
    protected function getRules()
    {
        return $this->rules;
    }

    /**
     * @var mixed
     */
    protected $value;

    public function isFillable()
    {
        return $this->getOption('fillable', function () {
            if ($this->getReadonlyOption() || $this->getDisabledOption()) {
                return false;
            }
            return true;
        });
    }

    public function setValue($value)
    {
        $this->value = $value;
    }

    public function getValue()
    {
        // forced value
        if ($this->value !== null) {
            return $this->value;
        }

        return $this->getForm()->getData($this->name);
    }

    public function addValidationRule($rule)
    {
        $this->rules[] = $rule;
    }

    public function getValidationRules()
    {
        $rules = [];

        if ($this->getRequiredOption()) {
            $rules[] = 'required';
        }

        $rules = array_merge($rules, $this->getRules());

        $formRules = $this->getOption('rules');
        if ($formRules !== null) {
            if (!is_array($formRules)) {
                $formRules = explode('|', $formRules);
            }

            foreach ($formRules as $rule) {
                $rules[] = $rule;
            }
        }

        return $rules;
    }

    public function getValidationName()
    {
        return $this->getName();
    }

    public function getValidationAttribute()
    {
        return $this->getOption('label', $this->name);
    }

    protected function getRequiredOption()
    {
        return $this->normalizeBoolean($this->getOption('required', false));
    }

    protected function getReadonlyOption()
    {
        return $this->normalizeBoolean($this->getOption('readonly', false));
    }

    protected function getDisabledOption()
    {
        return $this->normalizeBoolean($this->getOption('disabled', false));
    }

    /**
     * Modify value for displaying
     * @param $value
     * @return mixed
     */
    public function normalizeValue($value)
    {
        return $value;
    }

    /**
     * Parse boolean conditions
     * @param bool|string $required
     * @return bool
     * @throws \Exception
     */
    protected function normalizeBoolean($required)
    {
        if (is_bool($required)) {
            return $required;
        }
        elseif (is_string($required)) {
            list($condition, $conditionArgs) = explode(':', $required);
            $conditionArgsParts = explode(',', $conditionArgs);

            $fieldName = array_shift($conditionArgsParts);
            $fieldValue = $this->getForm()->getData($fieldName);

            switch ($condition)
            {
                case 'if':
                    if (count($conditionArgsParts) > 1) {
                        $result = in_array($fieldValue, $conditionArgsParts);
                    } elseif (count($conditionArgsParts) == 1) {
                        $result = $fieldValue == $conditionArgsParts[0];
                    } else {
                        $result = (bool)$fieldValue;
                    }
                    break;
                case 'unless':
                    if (count($conditionArgsParts) > 1) {
                        $result = !in_array($fieldValue, $conditionArgsParts);
                    } elseif (count($conditionArgsParts) == 1) {
                        $result = $fieldValue != $conditionArgsParts[0];
                    } else {
                        $result = !$fieldValue;
                    }
                    break;
                default:
                    throw new \Exception('Unknown required condition: '.$condition);
            }

            return $result;
        }
        else {
            return (bool)$required;
        }
        // TODO: is_callable
    }

    protected function viewData(array $options)
    {
        $data = parent::viewData($options);

        $value = $this->getOption('value', $this->getValue());
        $value = $this->normalizeValue($value);

        $required = $this->getRequiredOption();
        $readonly = $this->getReadonlyOption();
        $disabled = $this->getDisabledOption();

        return $data + compact('required', 'readonly', 'disabled', 'value');
    }
}