<?php

namespace AltSolution\Admin\Http\Controllers\Auth;

use AltSolution\Admin\Auth\TwoFactorAuthInterface;
use Illuminate\Http\Request;
use Illuminate\Auth\Events as LaravelEvents;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;

class F2AController extends Controller
{
    /**
     * @var TwoFactorAuthInterface
     */
    private $f2a;

    public function __construct(TwoFactorAuthInterface $f2a)
    {
        $this->f2a = $f2a;
    }

    public function verify()
    {
        if (!$this->f2a->hasUser()) {
            return redirect()->route('admin::login');
        }
        // TODO: replace with channel (sms, email, otp)
        $hasSecret = $this->f2a->requireSecret();

        $this->layout->setTitle(trans('admin::auth.f2a_title'));
        return view('admin::auth.f2a.verify', compact('hasSecret'));
    }

    public function verifyPost(Request $request)
    {
        if (!$this->f2a->hasUser()) {
            return redirect()->route('admin::login');
        }

        $throttleKey = $this->throttleKey($request);
        if (RateLimiter::tooManyAttempts($throttleKey, 5)) {
            event(new LaravelEvents\Lockout($request));

            $seconds = RateLimiter::availableIn($throttleKey);
            throw ValidationException::withMessages([
                'email' => trans('auth.throttle', [
                    'seconds' => $seconds,
                    'minutes' => ceil($seconds / 60),
                ]),
            ]);
        }

        $this->getValidationFactory()->extendImplicit('f2a', function ($attr, $value) use ($throttleKey) {
            $valid = $this->f2a->validate($value);
            if ($valid) {
                RateLimiter::clear($throttleKey);
            } else {
                RateLimiter::hit($throttleKey);
            }
            return $valid;
        });

        $this->validate($request, [
            'code' => 'required|f2a',
        ], [
            'f2a' => trans('admin::auth.f2a_invalid'),
        ]);

        $this->f2a->loginUser($this->guard());
        $this->f2a->forgetUser();

        return redirect()->intended($this->redirectPath());
    }

    /**
     * Get the guard to be used during authentication.
     *
     * @return \Illuminate\Contracts\Auth\StatefulGuard
     */
    protected function guard()
    {
        return Auth::guard();
    }

    /**
     * Get the post register / login redirect path.
     *
     * @return string
     */
    public function redirectPath()
    {
        return '/' . cms_admin_url();
    }

    /**
     * Get the rate limiting throttle key for the request.
     *
     * @return string
     */
    public function throttleKey(Request $request)
    {
        return $this->f2a->getUser()->getAuthIdentifier() . '|' . $request->ip();
    }
}
