<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Forms\CacheForm;
use Illuminate\Cache\CacheManager;
use Illuminate\Filesystem\Filesystem;
use Illuminate\Http\Request;

class CoreController extends Controller
{
    public function getCache()
    {
        $this->authorize('permission', 'cache');

        $form = cms_create_form(CacheForm::class, [
            'types' => ['cache', 'views'],
        ]);

        $this->layout
            ->setMenuSection('options')
            ->setTitle(trans('admin::core.cache_title'));
        return view('admin::core.cache', compact('form'));
    }

    public function postCache(Request $request)
    {
        $this->authorize('permission', 'cache');

        $form = cms_create_form(CacheForm::class);
        $form->validate($request);

        $types = $request->types;

        foreach ($types as $type) {

            if ($type == 'cache') {
                $ok = $this->clearCache();
                if ($ok) {
                    $this->layout->addNotify('success', trans('admin::core.cache_app_ok'));
                } else {
                    $this->layout->addNotify('error', trans('admin::core.cache_app_err'));
                }
            }

            if ($type == 'views') {
                $ok = $this->clearViews();
                if ($ok) {
                    $this->layout->addNotify('success', trans('admin::core.cache_view_ok'));
                } else {
                    $this->layout->addNotify('error', trans('admin::core.cache_view_err'));
                }
            }

            if ($type == 'config') {
                $this->authorize('permission', 'debug');

                $ok = $this->clearConfig();
                if ($ok) {
                    $this->layout->addNotify('success', trans('admin::core.cache_config_ok'));
                } else {
                    $this->layout->addNotify('error', trans('admin::core.cache_config_err'));
                }
            }

            if ($type == 'routes') {
                $this->authorize('permission', 'debug');

                $ok = $this->clearRoutes();
                if ($ok) {
                    $this->layout->addNotify('success', trans('admin::core.cache_route_ok'));
                } else {
                    $this->layout->addNotify('error', trans('admin::core.cache_route_err'));
                }
            }
        }

        return redirect()->route('admin::cache')->withInput();
    }

    /**
     * @see \Illuminate\Cache\Console\ClearCommand
     * @return bool
     */
    protected function clearCache(): bool
    {
        $app = app();
        /** @var CacheManager $cache */
        $cache = $app->make(CacheManager::class);
        /** @var Filesystem $files */
        $files = $app->make(Filesystem::class);

        $store = null;
        $tags = [];
        $app['events']->dispatch('cache:clearing', [$store, $tags]);

        $successful = $cache->store($store)->flush();

        if ($files->exists($storagePath = storage_path('framework/cache'))) {
            foreach ($files->files($storagePath) as $file) {
                if (preg_match('/facade-.*\.php$/', $file)) {
                    $files->delete($file);
                }
            }
        }

        if ($successful) {
            logger()->info('Cache cleared successfully.');
        } else {
            logger()->error('Failed to clear cache. Make sure you have the appropriate permissions.');
        }

        $app['events']->dispatch('cache:cleared', [$store, $tags]);

        return $successful;
    }

    protected function clearViews(): bool
    {
        $bladeOk = $this->clearBladeViews();
        $twigOk = $this->clearTwigViews();

        return $bladeOk && $twigOk;
    }

    /**
     * @see \Illuminate\Foundation\Console\ViewClearCommand
     * @return bool
     */
    protected function clearBladeViews(): bool
    {
        $app = app();
        /** @var Filesystem $files */
        $files = $app['files'];

        $successful = true;

        $path = $app['config']['view.compiled'];
        if ($path) {
            foreach ($files->glob("{$path}/*") as $view) {
                if (basename($view) == 'twig') {
                    // skip twig directory
                    continue;
                }
                $deleted = $files->delete($view);
                $successful = $successful && $deleted;
            }
        }

        if ($successful) {
            logger()->info('Blade cache cleaned');
        } else {
            logger()->error('Blade cache failed to be cleaned');
        }

        return $successful;
    }

    /**
     * @see \TwigBridge\Command\Clean
     * @return bool
     */
    protected function clearTwigViews(): bool
    {
        $app = app();
        /** @var \Twig\Environment $twig */
        $twig = $app['twig'];
        /** @var Filesystem $files */
        $files = $app['files'];

        $cacheDir = $twig->getCache();
        $files->deleteDirectory($cacheDir);

        $successful = !$files->exists($cacheDir);
        if ($successful) {
            logger()->info('Twig cache cleaned');
        } else {
            logger()->error('Twig cache failed to be cleaned');
        }

        return $successful;
    }

    /**
     * @see \Illuminate\Foundation\Console\ConfigClearCommand
     * @return bool
     */
    protected function clearConfig(): bool
    {
        $app = app();
        /** @var Filesystem $files */
        $files = $app['files'];

        $sucessful = true;
        if ($app->configurationIsCached()) {
            $deleted = $files->delete($app->getCachedConfigPath());
        }

        if ($sucessful) {
            logger()->info('Configuration cache cleared');
        } else {
            logger()->error('Configuration cache failed to be cleared');
        }

        return $sucessful;
    }

    /**
     * @see \Illuminate\Foundation\Console\RouteClearCommand
     * @return bool
     */
    protected function clearRoutes(): bool
    {
        $app = app();
        /** @var Filesystem $files */
        $files = $app['files'];

        $sucessful = true;
        if ($app->routesAreCached()) {
            $sucessful = $files->delete($app->getCachedRoutesPath());
        }

        if ($sucessful) {
            logger()->info('Route cache cleared');
        } else {
            logger()->error('Route cache failed to be cleared');
        }

        return $sucessful;
    }
}
