<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Image\ImagesInterface;
use AltSolution\Admin\Models\AclRole;
use AltSolution\Admin\Models\User;
use AltSolution\Admin\Modules\User\UserFormInterface;
use AltSolution\Admin\Modules\User\UserModelInterface;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;

class UsersController extends Controller
{
    public function getProfile(Request $request)
    {
        $this->authorize('permission', 'user.edit');
        $userId = $request->user()->id;
        return redirect()->route('admin::user_edit', $userId);
    }

    public function getIndex(Request $request)
    {
        $this->authorize('permission', 'user.list');

        $filterForm = cms_construct_filter(function () {
            return [
                'role' => [
                    'type' => 'selectModel',
                    'label' => trans('admin::user.role'),
                    'model' => AclRole::class,
                    'title_key' => 'name',
                    'nullable' => true,
                    'placeholder' => trans('admin::user.all_roles'),
                ],
            ];
        });
        $filterForm->setDataSource($request);
        $filter = $filterForm->extractData();

        /** @var Builder $qb */
        $qb = app(UserModelInterface::class)->query();
        if (!empty($filter['q'])) {
            $search = trim($filter['q']);
            $qb->where(function ($query) use ($search) {
                $query->where('name', 'LIKE', "%$search%")
                    ->orWhere('email', 'LIKE', "%$search%");
            });
        }
        if (!empty($filter['role'])) {
            $qb->where('acl_role_id', $filter['role']);
        }

        $items = $qb->orderBy('created_at', 'DESC')->paginate(config('admin.items_per_page', 20));
        $items->appends($filter);
        $current = auth()->user();

        $this->layout
            ->setMenuSection('users')
            ->setTitle(trans('admin::user.title'));

        return view('admin::users.list', compact('items', 'filterForm', 'current'));
    }

    public function getEdit($id = null)
    {
        $this->authorize('permission', 'user.edit');

        $user = null;
        if ($id) {
            $user = app(UserModelInterface::class)->query()->find($id);
        }
        $form = cms_create_form(UserFormInterface::class, $user);

        $this->layout
            ->setMenuSection('users')
            ->setTitle(trans($id ? 'admin::user.edit' : 'admin::user.add'))
            ->addBreadcrumb(trans('admin::user.title'), route('admin::user_list'));
        return view('admin::users.edit', compact('form'));
    }

    public function postSave(Request $request)
    {
        $this->authorize('permission', 'user.edit');

        $userId = $request->input('id');
        /** @var User $user */
        $user = app(UserModelInterface::class)->query()->firstOrNew(['id' => $userId]);
        $form = cms_create_form(UserFormInterface::class, $user);
        $validData = $form->validate($request, [], [
            'password.regex' => trans('admin::user.password_sux'),
        ]);

        $user->fill($validData);
        if ($user instanceof ImagesInterface) {
            $user->imageAllSave($request);
        }
        if ($request->password) {
    	   $user->password = bcrypt($request->password);
        }
        // don't allow user disable itself
        if ($user->id == auth()->user()->id) {
            $user->is_active = 1;
        }
        event('admin.model.user.saving', $user);
    	$user->save();
        event('admin.model.user.saved', $user);

    	$this->layout->addNotify('success', trans('admin::user.saved'));

        if ($request->button_apply) {
            return redirect()->route('admin::user_edit', ['id' => $user->id]);
        }

    	return redirect()->route('admin::user_list');
    }

    public function action(Request $request)
    {
        $action = $request->input('action');
        $ids = $request->input('ids');
        if (!$ids) {
            return;
        }

        /** @var User $user */
        if ($action == 'activate') {
            $this->authorize('permission', 'user.edit');

            foreach ($ids as $id) {
                $user = app(UserModelInterface::class)->query()->find($id);
                if (!$user) {
                    continue;
                }
                $user->is_active = 1;
                event('admin.model.user.saving', $user);
                $user->save();
                event('admin.model.user.saved', $user);
            }
        }
        elseif ($action == 'deactivate') {
            $this->authorize('permission', 'user.edit');

            $currentUserId = auth()->user()->id;
            foreach ($ids as $id) {
                $user = app(UserModelInterface::class)->query()->find($id);
                // do not allow deactivate himself
                if (!$user || $user->id == $currentUserId) {
                    continue;
                }
                event('admin.model.user.saving', $user);
                $user->is_active = 0;
                $user->save();
                event('admin.model.user.saved', $user);
            }
        }
        elseif ($action == 'delete') {
            $this->authorize('permission', 'user.delete');

            foreach ($ids as $id) {
                $user = app(UserModelInterface::class)->query()->find($id);
                if (!$user) {
                    continue;
                }
                if ($user instanceof ImagesInterface) {
                    $user->imageAllDelete();
                }
                // TODO: seo, uploads, etc
                event('admin.model.user.deleting', $user);
                $user->delete();
                event('admin.model.user.deleted', $user);
            }
        }
    }

}
