<?php

namespace AltSolution\Admin\Image;

use AltSolution\Admin\Image\ImageManagerInterface;
use Illuminate\Http\Request;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Arr;

/**
 * Use on models to add support for uploading images
 * @see ImagesInterface
 */
trait ImagesTrait
{
    /**
     * Return array of fields that consist of list of image sizes
     * @return array
     */
    abstract public function getImagesFields();

    /**
     * Return relative path where images were been stored
     * @return string
     */
    public function getImagesPath()
    {
        return $this->getTable();
    }

    /**
     * @return boolean|array
     */
    public function isImagesWebp()
    {
        return false;
    }

    // TODO: webp only setting

    public function imageSave($property, UploadedFile $file)
    {
        $manager = app(ImageManagerInterface::class);
        $sizes = Arr::get($this->getImagesFields(), $property);

        $fileName = $manager->generateFileName($file->getClientOriginalName());
        $stdFileName = $this->getStdFileName($fileName);
        $manager->store($file->getRealPath(), $stdFileName, $this->getImagesPath(), $sizes);

        if ($this->withWebp($property)) {
            $webpFileName = $this->getWebpFileName($stdFileName);
            $manager->store($file->getRealPath(), $webpFileName, $this->getImagesPath(), $sizes);
        }

        $this[$property] = $stdFileName;
    }

    public function imageSaveFile($property, \SplFileInfo $file)
    {
        if (!$file->isFile()) {
            throw new \Exception('Invalid file passed as parameter: ' . $file);
        }

        $manager = app(ImageManagerInterface::class);
        $sizes = Arr::get($this->getImagesFields(), $property);

        $fileName = $manager->generateFileName($file->getFilename());
        $stdFileName = $this->getStdFileName($fileName);
        $manager->store($file->getRealPath(), $stdFileName, $this->getImagesPath(), $sizes);

        if ($this->withWebp($property)) {
            $webpFileName = $this->getWebpFileName($stdFileName);
            $manager->store($file->getRealPath(), $webpFileName, $this->getImagesPath(), $sizes);
        }

        $this[$property] = $stdFileName;
    }

    public function imageSaveUrl($property, $url)
    {
        $manager = app(ImageManagerInterface::class);
        $sizes = Arr::get($this->getImagesFields(), $property);

        $file = $manager->downloadFile($url);
        $tempFile = $file->getRealPath();

        $fileName = $manager->generateFileName($file->getFilename());
        $stdFileName = $this->getStdFileName($fileName);
        $manager->store($tempFile, $stdFileName, $this->getImagesPath(), $sizes);

        if ($this->withWebp($property)) {
            $webpFileName = $this->getWebpFileName($stdFileName);
            $manager->store($tempFile, $webpFileName, $this->getImagesPath(), $sizes);
        }

        $file = null;
        @unlink($tempFile);
        //register_shutdown_function('unlink', $tempFile);

        $this[$property] = $stdFileName;
    }

    public function imageAllSave(Request $request)
    {
        $properties = $this->getImagesFields();
        foreach ($properties as $property => $sizes) {
            $file = $request->file($property);
            if ($file && $file->isValid()) {
                $this->imageDelete($property);
                $this->imageSave($property, $file);
            } elseif ($request[$property . '_delete']) {
                $this->imageDelete($property);
            } elseif (!$this->exists) {
                $this[$property] = '';
            }
        }
    }

    public function imageUrl($property, $size = null)
    {
        $manager = app(ImageManagerInterface::class);
        $stdFileName = $this[$property];
        if ($size !== null) {
            $sizes = Arr::get($this->getImagesFields(), $property);
            $params = Arr::get($sizes, $size);

            return $manager->generateUrl($stdFileName, $this->getImagesPath(), $size, $params);
        }

        return $manager->generateUrl($stdFileName, $this->getImagesPath());
    }

    public function imageWebpUrl($property, $size = null)
    {
        if (!$this->withWebp($property)) {
            return null;
        }

        $manager = app(ImageManagerInterface::class);
        $webpFileName = $this->getWebpFileName($this[$property]);
        if ($size !== null) {
            $sizes = Arr::get($this->getImagesFields(), $property);
            $params = Arr::get($sizes, $size);

            return $manager->generateUrl($webpFileName, $this->getImagesPath(), $size, $params);
        }

        return $manager->generateUrl($webpFileName, $this->getImagesPath());
    }

    // TODO: imageFile()

    public function imageDelete($property)
    {
        $manager = app(ImageManagerInterface::class);
        $stdFileName = $this->getOriginal($property);
        if ($stdFileName) {
            $sizes = Arr::get($this->getImagesFields(), $property);
            $manager->delete($stdFileName, $this->getImagesPath(), $sizes);
            if ($this->withWebp($property)) {
                $webpFileName = $this->getWebpFileName($stdFileName);
                $manager->delete($webpFileName, $this->getImagesPath(), $sizes);
            }
        }
        $this[$property] = '';
    }

    public function imageAllDelete()
    {
        $manager = app(ImageManagerInterface::class);
        foreach ($this->getImagesFields() as $property => $sizes) {
            $stdFileName = $this->getOriginal($property);
            if ($stdFileName) {
                $manager->delete($stdFileName, $this->getImagesPath(), $sizes);
                if ($this->withWebp($property)) {
                    $webpFileName = $this->getWebpFileName($stdFileName);
                    $manager->delete($webpFileName, $this->getImagesPath(), $sizes);
                }
            }
            $this[$property] = '';
        }
    }

    public function imageReset($property)
    {
        $manager = app(ImageManagerInterface::class);
        $stdFileName = $this->getOriginal($property);
        $sizes = Arr::get($this->getImagesFields(), $property);
        $manager->reset($stdFileName, $this->getImagesPath(), $sizes);
        if ($this->withWebp($property)) {
            $webpFileName = $this->getWebpFileName($stdFileName);
            $manager->reset($webpFileName, $this->getImagesPath(), $sizes);
        }
    }

    public function imageAllReset()
    {
        $manager = app(ImageManagerInterface::class);
        foreach ($this->getImagesFields() as $property => $sizes) {
            $stdFileName = $this->getOriginal($property);
            $manager->reset($stdFileName, $this->getImagesPath(), $sizes);
            if ($this->withWebp($property)) {
                $webpFileName = $this->getWebpFileName($stdFileName);
                $manager->reset($webpFileName, $this->getImagesPath(), $sizes);
            }
        }
    }


    private function withWebp($property)
    {
        $isWebp = $this->isImagesWebp();
        if (is_bool($isWebp)) {
            return $isWebp;
        }
        $isPropWebp = Arr::get($isWebp, $property, false);
        // TODO: control each size individual
        return $isPropWebp;
    }

    private function getStdFileName(string $oldFileName): string
    {
        $filename = pathinfo($oldFileName, PATHINFO_FILENAME);
        $ext = pathinfo($oldFileName, PATHINFO_EXTENSION);
        if ($ext === 'webp') {
            $ext = 'png';
        }
        return "{$filename}.{$ext}";
    }

    private function getWebpFileName(string $oldFileName): string
    {
        $filename = pathinfo($oldFileName, PATHINFO_FILENAME);
        //$ext = pathinfo($oldFileName, PATHINFO_EXTENSION);
        return "{$filename}.webp";
    }
}
