<?php

namespace AltSolution\Admin\Providers;

use AltSolution\Admin\Http\Controllers\CronController;
use AltSolution\Admin\Cron\Repository;
use AltSolution\Admin\Cron\RepositoryInterface;
use AltSolution\Admin\Console\Commands\ListSchedule;
use AltSolution\Admin\Models\Cron;
use Carbon\Carbon;
use Illuminate\Support\ServiceProvider;
use Illuminate\Console\Scheduling\Schedule;
use Illuminate\Support\Str;

class CronProvider extends ServiceProvider
{
    protected $defer = true;

    public function provides()
    {
        return [
            'admin.controller.cron',
            RepositoryInterface::class,
        ];
    }

    public function register()
    {
        $this->app->instance('admin.controller.cron', CronController::class);
        $this->app->singleton(RepositoryInterface::class, Repository::class);

        if ($this->app->runningInConsole()) {
            $this->commands([
                ListSchedule::class,
            ]);
        }
    }

    /**
     * Register any services.
     *
     * @return void
     */
    public function boot()
    {
        $this->app->resolving(Schedule::class, function ($schedule) {
            // runs only on console kernel boot
            $this->schedule($schedule);
        });
    }

    /**
     * @param Schedule $schedule
     */
    private function schedule(Schedule $schedule)
    {
        /** @var RepositoryInterface $repo */
        $repo = $this->app->make(\AltSolution\Admin\Cron\RepositoryInterface::class);
        $jobs = $repo->getAll();

        $jobs->each(function ($job) use ($schedule, $repo) {
            /** @var Cron $job */
            if ($job->type == Cron::TYPE_ARTISAN) {
                $commandParts = explode(' ', $job->command . ' --no-ansi', 2);
                if (count($commandParts) > 1) {
                    $event = $schedule->command($commandParts[0], $this->compileParameters($commandParts[1], true));
                } else {
                    $event = $schedule->command($commandParts[0]);
                }
            } elseif ($job->type == Cron::TYPE_SHELL) {
                $event = $schedule->exec($job->command);
            } else {
                return;
            }
            $started = null;
            $event->cron($job->expression)
                ->description($job->description)
                ->before(function () use ($job, &$started) {
                    $started = Carbon::now();
                })
                ->then(function () use ($job, $repo, &$started, $event) {
                    $ended = Carbon::now();
                    $outputPath = storage_path($job->getOutputPath());
                    if (is_readable($outputPath)) {
                        $output = file_get_contents($outputPath);
                        @unlink($outputPath);
                    } else {
                        $output = '';
                    }
                    $repo->addLog($job, $started, $ended, $output);
                })
                ->sendOutputTo(storage_path($job->getOutputPath()));
            if ($job->in_maintenance) {
                $event->evenInMaintenanceMode();
            }
            if (!$job->can_overlap) {
                $event->withoutOverlapping();
            }
            //$event->onOneServer();
        });
    }

    private function compileParameters($parameters, $console = false)
    {
        if ($parameters) {
            $regex = '/(?=\S)[^\'"\s]*(?:\'[^\']*\'[^\'"\s]*|"[^"]*"[^\'"\s]*)*/';
            preg_match_all($regex, $parameters, $matches, PREG_SET_ORDER, 0);
            $argument_index = 0;
            $result = collect($matches)->mapWithKeys(function ($parameter) use ($console, &$argument_index) {
                $param = explode('=', $parameter[0]);
                if (count($param) > 1) {
                    $trimmed_param = trim(trim($param[1], '"'), "'");
                    if ($console) {
                        return Str::startsWith($param[0], '--') ? [$param[0] => $trimmed_param] : [$argument_index++ => $trimmed_param];
                    }
                    return [$param[0] => $trimmed_param];
                }
                return Str::startsWith($param[0], '--') && ! $console ? [$param[0] => true] : [$argument_index++ => $param[0]];
            })->toArray();
            return $result;
        }
        return [];
    }
}
